#ifndef _MODULES_H_

#define _MODULES_H_

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <unistd.h>
#include <netdb.h>
#include <dlfcn.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <errno.h>
#include <dirent.h>
#include <setjmp.h>
#include "chunkyircd.h"
#include "utility.h"
#include "server.h"
#include "client.h"
#include "ircd.h"
#include "parse.h"

#ifdef USING_OPENBSD
#define BROKEN_DLOPEN
#define DL_FLAGS DL_LAZY
#define VERSION_SYMBOL "_version"
#define FLAGS_SYMBOL "_flags"
#else
#define DL_FLAGS RTLD_NOW
#define VERSION_SYMBOL "version"
#define FLAGS_SYMBOL "flags"
#endif

#define CMIRCD_MODULE(x,y) float version = (x); \
			   int flags = (y)

/* Possible Module Types */
#define CMD_CLIENT 	1	/* Client Command */
#define CMD_SERVER	2	/* Server Command */

/* Module information */
struct _modinfo {
	char name[30];
	long int use_count;
	void *handle;
	struct _modinfo *next;
	int flags;
	float version;
};

typedef struct _modinfo *ModInfo;

int ircd_module_init();
int ircd_module_unload(char *);
int ircd_module_unload_all();
int ircd_module_disable(char *);
int ircd_module_rehash();
int ircd_module_dlopen(ClientInfo, ServerInfo, Cmd);

/* Commands that cannot be modularised */
int cmd_unload(ClientInfo, ServerInfo, Cmd);
int cmd_rehash(ClientInfo, ServerInfo, Cmd);

void caught_segfault(int);

#endif
