/* This file is part of Chunky Monkey IRCD 
 *
 * Chunky Monkey IRCD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Chunky Monkey IRCD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Chunky Monkey IRCD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $Id: cmd_list.c,v 1.8.4.4 2002/09/06 19:52:46 lloydy Exp $
 *
 */
#include "chunkyircd.h"
#include "network.h"
#include "channel.h"

CMIRCD_MODULE(1.02, CMD_CLIENT | CMD_SERVER);

static int send_list(ClientInfo, ChannelInfo);

/*
* LIST - list the channels we know about
* according to the variables they set
*/
int cmd_list(ClientInfo c_ptr, ServerInfo s_ptr, Cmd cmd)
{
    int max = 9999;
    int min = -1;
    ChannelInfo chan = NULL;
    int i = 0;
    char *chan_name = NULL;
    char buffer[BUF_SIZE];

    if (s_ptr) {

	return 0;
    }

    for (i = 0; cmd->params[i] != NULL; i++) {
	/* Set limits if we have right char */
	switch (cmd->params[i][0]) {

	case '>':
	    min = atoi(&(cmd->params[i][1]));
	    break;

	case '<':
	    max = atoi(&(cmd->params[i][1]));
	    break;

	case '#':
	case '&':
	    chan_name = cmd->params[i];
	    break;

	default:
	    break;
	}
    }

    if (chan_name) {
	/* Locate The channel they asked for */
	chan = locate_channel(chan_name);

	if (chan) {
	    /* Do that crazy list thing */
	    send_list(c_ptr, chan);
	    send_numeric(c_ptr, get_my_server(), 323, ":End of /LIST");
	} else {
	    ircd_sprintf(buffer, BUF_SIZE,
			 "%s :No such channel", chan_name);
	    send_numeric(c_ptr, get_my_server(), 403, buffer);
	}
    } else {
	chan = get_channel_with_mask("*");
	while (chan) {
	    if (!(chan->mode & CMODE_SECRET)) {
		if (chan->user_count < max && chan->user_count > min) {
		    send_list(c_ptr, chan);
		}
	    }
	    chan = get_channel_with_mask(NULL);
	}
	send_numeric(c_ptr, get_my_server(), 323, ":End of /LIST");
    }


    return 1;
}

/*
* Send a names list for a given 
* channel
*/
static int send_list(ClientInfo c_ptr, ChannelInfo chan)
{
    char buffer[BUF_SIZE];

    ircd_sprintf(buffer, BUF_SIZE, "%s %ld :%s", chan->name,
		 chan->user_count, chan->topic);

    send_numeric(c_ptr, get_my_server(), 322, buffer);

    return 1;
}
