/* This file is part of Chunky Monkey IRCD 
 *
 * Chunky Monkey IRCD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Chunky Monkey IRCD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Chunky Monkey IRCD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $Id: cmd_pong.c,v 1.7.4.15 2002/09/06 19:52:47 lloydy Exp $
 *
 */
#include "modules.h"

CMIRCD_MODULE(1.02, CMD_CLIENT | CMD_SERVER);

/* 
 * respond to a pong
 */
int cmd_pong(ClientInfo c_ptr, ServerInfo s_ptr, Cmd cmd)
{
    char *origin = NULL;
    char *target = NULL;
    char buffer[BUF_SIZE];
    ServerInfo my_serv = NULL;

    my_serv = get_my_server();

    /* Account for single param being PONG :param */
    if (!check_params(cmd, 1) && !cmd->text) {
	if (c_ptr) {
	    send_numeric(c_ptr, get_my_server(), 409,
			 ":No origin specified");
	}
	return -1;
    }

    /* Get the origin */
    origin = cmd->params[0] ? cmd->params[0] : cmd->text;
    if (!origin) {
	if (c_ptr) {
	    send_numeric(c_ptr, get_my_server(), 409,
			 ":No origin specified");
	}
	return -1;
    }

    /* Get the target (Not essential) */
    target = cmd->params[1];

    if (!(target) || !(lc_strncmp(target, my_serv->name, -1))) {
	/* Its for us */
	if (c_ptr) {
	    c_ptr->flags &= ~USR_PINGSENT;
	} else {
	    s_ptr->flags &= ~USR_PINGSENT;
	}
    } else {
	ServerInfo dest_server = NULL;
	ClientInfo dest_client = NULL;

	dest_server = server_locate(target);
	dest_client = client_locate(target);

	ircd_sprintf(buffer, BUF_SIZE, "PONG %s %s", origin, target);

	if (dest_server) {
	    if (c_ptr) {
		send_origin_server(dest_server, c_ptr->nick, buffer);
	    }
	    if (s_ptr && cmd->origin) {
		send_origin_server(dest_server, cmd->origin, buffer);
	    }
	}
	if (dest_client) {
	    if (c_ptr) {
		send_origin_client(dest_client, c_ptr->nick, buffer);
	    }
	    if (s_ptr && cmd->origin) {
		send_origin_client(dest_client, cmd->origin, buffer);
	    }
	}

    }

    return 0;
}
