/* This file is part of Chunky Monkey IRCD 
 *
 * Chunky Monkey IRCD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Chunky Monkey IRCD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Chunky Monkey IRCD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $Id: cmd_svsmode.c,v 1.1.2.11 2002/09/06 19:52:47 lloydy Exp $
 *
 */
#include "modules.h"

static void svsmode_change(ClientInfo, ServerInfo, char *, char *);

CMIRCD_MODULE(1.02, CMD_SERVER);

/*
 * Used by Services to set user modes
 * Can safely assume we won't be called with server=NULL
 */
int cmd_svsmode(ClientInfo user_info, ServerInfo server, Cmd cmd)
{
    char *mline = NULL;		/* Mode can be in text or params[2] */
    ClientInfo subject = NULL;
    ClientInfo c_origin = NULL;
    ServerInfo s_origin = NULL;

    /* Set mline to point to the correct string */
    mline = cmd->text ? cmd->text : cmd->params[1];

    if (!mline) {
	return 0;
    }

    /* Get the person we're talking about */
    if (!(cmd->params[0])
	|| !(subject = client_locate(cmd->params[0]))) {
	/* No subject */
	return 0;
    } else {
	if (cmd->origin) {
	    if ((c_origin = client_locate(cmd->origin))) {
		svsmode_change(subject, server, c_origin->nick, mline);
	    }
	    if ((s_origin = server_locate(cmd->origin))) {
		svsmode_change(subject, server, s_origin->name, mline);
	    }
	} else {
	    svsmode_change(subject, server, server->name, mline);
	}
    }

    return 1;
}

/*
 * Change the mode in question
 */
static void svsmode_change(ClientInfo subject,
			   ServerInfo src, char *origin, char *mline)
{
    char mode_buff[10];
    char buffer[BUF_SIZE];
    char *mode_pos = mode_buff;
    int givetake = 0;

    memset(mode_buff, 0, 10);

    while (*(mline)) {

	switch (*mline) {

	case '+':
	    if (givetake <= 0) {
		givetake = 1;
		*(mode_pos++) = '+';
	    }
	    break;

	case '-':
	    if (givetake >= 0) {
		givetake = -1;
		*(mode_pos++) = '-';
	    }
	    break;

	case 'r':
	    if ((givetake > 0)
		&& !(subject->mode & MODE_REGIST)) {
		subject->mode |= MODE_REGIST;
		*(mode_pos++) = 'r';
	    }
	    if ((givetake < 0)
		&& (subject->mode & MODE_REGIST)) {
		subject->mode &= ~(MODE_REGIST);
		*(mode_pos++) = 'r';
	    }
	    break;

	default:
	    break;

	}
	mline++;
    }

    if (mode_buff[1] != '\0') {

	/* Send it */
	ircd_sprintf(buffer, BUF_SIZE, "MODE %s %s", subject->nick,
		     mode_buff);
	send_origin_client(subject, origin, buffer);

	/* Send to servers */
	ircd_sprintf(buffer, BUF_SIZE, "SVSMODE %s :%s",
		     subject->nick, mode_buff);
	servers_relay(src, origin, buffer);
    }

}
